/*
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
************************** 504_Programm_n_Fakultaet ***************************
-------------------------------------------------------------------------------
******************************* main.c ** V1.1 ********************************
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
 Berechnung einer n-Fakultaet
 Zusaetzliche Header..: keine
 Module.(*.c und *.h).: application, lcd_2wire, num_conversion
 Benoetigte Hardware..: Testboard, 20x4-LCD, 2-Draht-LCD-Interface
 Version..............: 1.1
 Compiler.............: CodeVisionAVR
 Chip.................: ATmega88
 Datum................: Juni 2009
 Autor................: Alwin Lenck (ALE23) basierend auf dem Projekt 
                        2-Draht-LCD von Udo Juerss
-------------------------------------------------------------------------------
Ein typisches Beispiel fuer eine rekursive C-Funktionen ist die Berechnung
einer mathematischen Fakultaet, die selbst eine primitive rekursive
mathematische Funktion ist; zum Beispiel:

      5! = 1 * 2 * 3 * 4 * 5 = 120

Das kann mathematisch ausgedrueckt werden mit:

      n! = 1 * 2 *  ... * (n-3) * (n-2) * (n-1)    oder
      n! = n * (n-1)!

Ausgabe von n! auf dem 20x4-LCD

Anschluesse des 20x4-LCD:
CLOCK................: PB1 (kann geaendert werden in application.h)       *****
DATA.................: PB2 (kann geaendert werden in application.h)       *****
*/

//-----------------------------------------------------------------------------
// Header-Dateien einfuegen:
//-----------------------------------------------------------------------------
#include "application.h"     // Initialisierung der Anwendung             *****
#include "lcd_2wire.h"       // Initialisierung des 2Draht-LCDs           *****
#include "num_conversion.h"  // Initialisierung der Konvertierung         *****

//-----------------------------------------------------------------------------
// Funktions-Prototypen:
//-----------------------------------------------------------------------------
int fakult(int);             // Funktion fakult(n)

//-----------------------------------------------------------------------------
// Globale Variablen:
//-----------------------------------------------------------------------------
__flash char string[21] =    // String fuer Kopfzeile im FLASH-Speicher   *****
"n! = xxxx fuer n = i";      // Definition der Kopfzeile                  *****
__flash char *p;             // Pointer auf Character im FLASH-Speicher   *****
int c;                       // Zeichen-Zaehler fuer die Kopfzeile        *****

int n;                       // n ist die Ergebnis-Groesse
int i = 5;                   // i ist die zu berechnende Fakultaet
                             // Zur Berechnung anderer Fakultaeten kann die 
                             // Zahl 5 hier geaendert werden.

//-----------------------------------------------------------------------------
// Hauptprogramm main
//-----------------------------------------------------------------------------
void main(void)
{
// Die folgenden 2 Anweisungen stammen vom AVR-Projekt 2-Draht-LCD        *****
  // Setzen des System-Taktes (CPU-Clock),                                *****
  // Ausschalten nicht benutzter Hardware-Module                          *****
  app_init();                // Initialisierung der Anwendung             *****

  // Definition von Prototypen fuer LCD-Ausgabe-Funktionen usw.           *****
  lcd_init();                // Initialisierung des LCD-Interfaces        *****

                             // In n steht spaeter das Ergebnis:
  n = fakult(i);             // Funktions-Aufruf; berechne 5! rekursiv 

  lcd_goto_xy(0,0);          // Cursor an den Anfang des LCD              *****
                             // Spalte x = 0 und Zeile y = 0              *****
  p = string;                // identisch mit p = &string[0]              *****
  for(c = 0; c < 20; c++)    // LCD-Kopfzeile Zeichen fuer Zeichen        *****
     lcd_write_char(*p++);   // ausgeben und jeweils auf naechstes        *****
                             // Zeichen im String zeigen.                 *****
  lcd_goto_xy(5,0);          // Cursor auf x = 5 und Zeile y = 0          *****
     lcd_write_S16(n,4);     // Ergebnis von n! mit 4 Ziffern ausgeben    *****
  lcd_goto_xy(19,0);         // Cursor auf x = 19 und Zeile y = 0         *****
     lcd_write_S16(i,1);     // Zu berechnende Fakultaet max. 1 Ziffer    *****

  while(1)                   // Die Arbeit ist getan; Warteschleife 
  ;                          // ohne weitere Anweisungen
}

int fakult(int n)            // Rekursive Funktion fakult(n)
{
  if(n == 0)                 // Wenn n = 0 ist: Uebergebe fuer 0! eine 1 an
  return 1;                  // die aufrufende Stelle (gem. Definition).
  else                       // Andernfalls rufe dich solange 
  return (n * fakult(n-1));  // mit jeweils (n-1) selbst auf,
                             // bis (n-1) = 0 erreicht ist.
}
//-----------------------------------------------------------------------------
